function Make_Movie(fh, dummy)
global N_constr N_max
global SINKS Omega1 Omega2 Omega3 Omega4  Ar A1r A2r A3r A4r
disp(' ');
disp('The FSP code is running.')

handles = guidata(fh);
tau = handles.tau;
Tarray = [linspace(0,tau,50)];

N = [handles.Xmax1,handles.Xmax2];

init_cond = handles.XInit*(N(2)+1)+handles.YInit+1;

N = [handles.Xmax1,handles.Xmax2];
N_max = handles.N_max;
N_constr = length(N_max);

[A1r,A2r,A3r,A4r,Iin2,handles.Hits] = GetA_From_GUI_Time_Vary;
SINKS = [length(A1r)-N_constr+1:length(A1r)];
Omega1 = handles.Omega1;
Omega2 = handles.Omega2;
Omega3 = handles.Omega3;
Omega4 = handles.Omega4;

P0 = spalloc((N(1)+1)*(N(2)+1),1,1);
P0(init_cond)=1;
P0 = [P0(Iin2);zeros(length(SINKS),1)];

options = odeset('Jacobian',@CME_Jacobian,'JPattern',find(A1r+A2r+A3r+A4r),'AbsTol',1e-7);
[TOUT,P_arr] = ode15s(@CME_dot,Tarray,P0,options);  %solve CME at given time points

options = odeset('AbsTol',1e-7);
[TOUT,MAK_Response] = ode15s(@MAK_dot,Tarray,[handles.XInit;handles.YInit],options);  %solve CME at given time points

P_arr =P_arr';
P_array = spalloc((N(1)+1)*(N(2)+1),length(TOUT),length(TOUT)*length(Iin2));
P_array(Iin2,:) = P_arr(1:(length(A1r)-N_constr),:);

h_mov = handles.Movie_Fig;
figure(h_mov);
clf
clear M
for i_time = 1:length(Tarray)
    Z = spalloc(N(1)+1,N(2)+1,0);
        P2_con = P_array(:,i_time);
    for i=0:N(1);
        for j=0:N(2)
            ix = i*(N(2)+1)+j+1;  % index of current state in rectangle.
            Z(i+1,j+1)=P2_con(ix);
        end
    end
    Z= max(Z,0);
    subplot(2,2,1);  cla
    if get(handles.MeshBox,'Value')
        mesh([0:N(1)],[0:N(2)],(Z'));
        axis([0 handles.Xmax1 0 handles.Xmax2 0 max(max(Z))])
        xlabel('Species X','fontsize',12)
        ylabel('Species Y','fontsize',12)
        zlabel('Probability','fontsize',12)
    else
        contourf([N(1)-1,N(1)],[N(2)-1,N(2)],[0.06 0;0 0.06]);
        hold on
        contourf([0:N(1)],[0:N(2)],Z');
        axis([0 handles.Xmax1 0 handles.Xmax2])
        xlabel('Species X','fontsize',12)
        ylabel('Species Y','fontsize',12)
        title('Joint Probability Distribution','fontsize',12)
    end

    h=subplot(2,2,3);
    plot([0:N(1)],sum(Z,2),handles.col,'linewidth',3);
    xlabel('Number of Species X','fontsize',12);
    ylabel('Probability Density','fontsize',12)
    title('Marginal Distribution of X','fontsize',12)
    set(h,'xlim',[0 handles.Xmax1])
    mean1(i_time) = [0:N(1)]*(sum(Z,2));
    var1(i_time)  = ([0:N(1)].^2)*(sum(Z,2))-mean1(i_time)^2;

    h=subplot(2,2,4);
    plot([0:N(2)],sum(Z),handles.col,'linewidth',3);
    xlabel('Number of Species Y','fontsize',12);
    ylabel('Probability Density','fontsize',12)
    title('Marginal Distribution of Y','fontsize',12)
    set(h,'xlim',[0 handles.Xmax2])
    mean2(i_time) = [0:N(2)]*(sum(Z))';
    var2(i_time)  = ([0:N(2)].^2)*(sum(Z))'-mean2(i_time)^2;


    subplot(2,2,2);
    cla
    plot(Tarray(1:i_time),mean2,'linewidth',3)
    hold on
    plot(Tarray(1:i_time),MAK_Response(1:i_time,2),'g--','linewidth',3)
    h=gca;
    set(h,'xlim',[0,tau])
    xlabel('Time','fontsize',12)
    ylabel('Mean of species Y','fontsize',12)
    title('-Stochastic; --Deterministic','fontsize',12)

    drawnow
    M(i_time) = getframe(h_mov);

end
movie2avi(M,'temp.avi')

axes(handles.SSA_XY);
plot(mean1,mean2,'k--',MAK_Response(1:i_time,1),MAK_Response(1:i_time,2),'g')

axes(handles.SSA_X);
plot(Tarray,mean1,'k--',Tarray,MAK_Response(1:i_time,1),'g')

axes(handles.SSA_Y);
plot(Tarray,mean2,'k--',Tarray,MAK_Response(1:i_time,2),'g')



end
function [Pdot] = CME_dot(t,P)
global  Omega1 Omega2 Omega3 Omega4  Ar A1r A2r A3r A4r
Pdot = Ar*P;
if Omega1~=0
    Pdot=Pdot+real(exp(-i*Omega1*t-i*pi/2))*A1r*P;
end
if Omega2~=0
    Pdot=Pdot+real(exp(-i*Omega2*t-i*pi/2))*A2r*P;
end
if Omega3~=0
    Pdot=Pdot+real(exp(-i*Omega3*t-i*pi/2))*A3r*P;
end
if Omega4~=0
    Pdot=Pdot+real(exp(-i*Omega4*t-i*pi/2))*A4r*P;
end
end
function [Jac] = CME_Jacobian(t,P)
global  Omega1 Omega2 Omega3 Omega4  Ar A1r A2r A3r A4r
Jac = Ar;
if Omega1~=0
    Jac=Jac+real(exp(-i*Omega1*t-i*pi/2))*A1r;
end
if Omega2~=0
    Jac=Jac+real(exp(-i*Omega2*t-i*pi/2))*A2r;
end
if Omega3~=0
    Jac=Jac+real(exp(-i*Omega3*t-i*pi/2))*A3r;
end
if Omega4~=0
    Jac=Jac+real(exp(-i*Omega4*t-i*pi/2))*A4r;
end
end
function [fdot] = MAK_dot(t,x)
handles = guidata(1);
fdot = [handles.Prop1(x(1),x(2))*(1+real(exp(-i*handles.Omega1*t-i*pi/2)))-...
    handles.Prop2(x(1),x(2))*(1+real(exp(-i*handles.Omega2*t-i*pi/2)));...
    handles.Prop3(x(1),x(2))*(1+real(exp(-i*handles.Omega3*t-i*pi/2)))-...
    handles.Prop4(x(1),x(2))*(1+real(exp(-i*handles.Omega4*t-i*pi/2)))];
end


