function [handles]= SSA_Callback(handles)
disp(' ');
disp('An SSA code is being run.')
% handles = guidata(1);
tau = handles.tau;

if max(abs([handles.Omega1,handles.Omega2,handles.Omega3,handles.Omega4]))~=0
    disp('This version of SSA handles time varying Rate Constant.')
    disp('by adding a no-change reaction with rate 10*max(abs(Omega_i)).')
    disp('This is an approximation.')
end

%% run a SSA loop until final time or until 1000 events have been
%% recorded.
cnt_max = 200000;
xx = zeros(cnt_max,2);
tt = zeros(1,cnt_max);

i_count = 0;
t=0;
x = [handles.XInit;handles.YInit];
xx(1,:)=[x'];
while t<tau&&i_count<cnt_max
    if handles.File==0
        w = [handles.Prop1(x(1),x(2)),...
            handles.Prop2(x(1),x(2)),...
            handles.Prop3(x(1),x(2)),...
            handles.Prop4(x(1),x(2))];
        S = [1 -1 0 0;...
            0 0 1 -1];
    else
        [S,w] = handles.PropFunct(x(1),x(2));
    end
    if max(abs([handles.Omega1,handles.Omega2,handles.Omega3,handles.Omega4]))~=0
        w(1) = (1+real(exp(-j*handles.Omega1*t-j*pi/2)))*w(1);
        w(2) = (1+real(exp(-j*handles.Omega2*t-j*pi/2)))*w(2);
        w(3) = (1+real(exp(-j*handles.Omega3*t-j*pi/2)))*w(3);
        w(4) = (1+real(exp(-j*handles.Omega4*t-j*pi/2)))*w(4);
        w(5) = 10*max(abs([handles.Omega1,handles.Omega2,handles.Omega3,handles.Omega4]));
        S = [S,[0;0]];
    end
   
    w0 = sum(w);            % compute the sum of the prop. functions
    t = t+1/w0*log(1/rand);  % update time of next reaction

    if t>tau
        xx =  [xx(1:i_count,:);x'];
        tt = [tt(1:i_count),tau];
       break
    end

    r2w0=rand*w0;           % generate second random number and multiply by prop. sum

    i=1;                    % initialize reaction counter
    while sum(w(1:i))<r2w0  % increment counter until sum(w(1:i)) exceeds r2w0
        i=i+1;
    end
    
    i_count=i_count+1;
    xx(i_count,:)=x';
    tt(i_count)=t;
    x = x+S(:,i);           % update the configuration
    i_count=i_count+1;
    xx(i_count,:)=x';
    tt(i_count)=t;

end
handles.SSA_XY_Counts =  xx;
handles.SSA_Time_Points= tt;

% guidata(fh,handles);

% if max(abs([handles.Omega1,handles.Omega2,handles.Omega3,handles.Omega4]))==0

%     axes(handles.SSA_XY);
%     plot(xx(:,1),xx(:,2),[handles.col,'o'])
%     hold on
%     xlabel('Species X','fontsize',12)
%     ylabel('Species Y','fontsize',12)
%     title('SSA Trajectory in X-Y Plane')
% 
%     axes(handles.SSA_X);
%     plot(tt,xx(:,1),handles.col)
%     hold on
%     xlabel('Time','fontsize',12)
%     ylabel('Species X','fontsize',12)
%     title('Trajectory of X')
% 
%     axes(handles.SSA_Y);
%     plot(tt,xx(:,2),handles.col)
%     hold on
%     xlabel('Time','fontsize',12)
%     ylabel('Species Y','fontsize',12)
%     title('Trajectory of Y')
% 
% end
disp(' ');
disp('The SSA is Complete.')

