function handles = Solve_Callback(handles)
global N_constr N_max
global SINKS Omega1 Omega2 Omega3 Omega4  Ar A1r A2r A3r A4r Epsilon
%        handles = guidata(fh);
tau = handles.tau;
N = [handles.Xmax1,handles.Xmax2];
init_cond = handles.XInit*(N(2)+1)+handles.YInit+1;

N_max = handles.N_max;
N_constr = length(N_max);

[A1r,A2r,A3r,A4r,Iin2,handles.Hits] = GetA_From_GUI_Time_Vary(handles); % GetA_From_GUI_2(N,Poly_Proj);
Ar = A1r+A2r+A3r+A4r;
SINKS = [length(Ar)-N_constr+1:length(Ar)];

P0 = spalloc((N(1)+1)*(N(2)+1),1,1);
P0(init_cond)=1;
P0 = [P0(Iin2);zeros(length(SINKS),1)];


if max(abs([handles.Omega1,handles.Omega2,handles.Omega3,handles.Omega4]))==0
    %% Time invariant system, use mexpv solver.
    tryagain=1;
    m_p = 20;
    while tryagain==1
        [P2, err, hump, Time_array, P_array, P_lost, tryagain] = mexpv_modified_05032010(tau,Ar,P0,1e-14,m_p,1,tau,handles.Epsilon,SINKS);
        m_p = m_p+10;
    end
else
    %% Time varying system -- use ODE solver
    Omega1 = handles.Omega1;
    Omega2 = handles.Omega2;
    Omega3 = handles.Omega3;
    Omega4 = handles.Omega4;
    
    if Omega1==0
        A1r=0;
    end
    if Omega2==0
        A2r=0;
    end
    if Omega3==0
        A3r=0;
    end
    if Omega4==0
        A4r=0;
    end
    
    Epsilon = handles.Epsilon;
    options = odeset('Jacobian',@CME_Jacobian,'JPattern',find(Ar),'AbsTol',1e-7);
    [TOUT,P_arr] = ode15s(@CME_dot,[0 tau/2 tau],P0,options);  %solve CME at given time points
    
    P2 = P_arr(3,:)';
    P_lost = P2(SINKS);
    
end
P2_con = spalloc((N(1)+1)*(N(2)+1),1,length(Iin2));
P2_con(Iin2) = P2(1:length(A1r)-N_constr);

handles.err =sum(P_lost);

[J,I] = find(ones(N(2)+1,N(1)+1));
handles.Bounds = P_lost;

handles.Z = sparse(I(Iin2),J(Iin2),P2_con(Iin2));
handles.Z(N(1)+1,N(2)+1)=0;

handles.Z= max(handles.Z,0);
%         guidata(1,handles);
end
function [Pdot] = CME_dot(t,P)
global  Omega1 Omega2 Omega3 Omega4  Ar A1r A2r A3r A4r SINKS Epsilon
if sum(P(SINKS))<Epsilon*2
    Pdot = Ar*P;
    if Omega1~=0
        Pdot=Pdot+real(exp(-i*Omega1*t-i*pi/2))*A1r*P;
    end
    if Omega2~=0
        Pdot=Pdot+real(exp(-i*Omega2*t-i*pi/2))*A2r*P;
    end
    if Omega3~=0
        Pdot=Pdot+real(exp(-i*Omega3*t-i*pi/2))*A3r*P;
    end
    if Omega4~=0
        Pdot=Pdot+real(exp(-i*Omega4*t-i*pi/2))*A4r*P;
    end
else
    Pdot = 0*P;
end
end
function [Jac] = CME_Jacobian(t,P)
global  Omega1 Omega2 Omega3 Omega4  Ar A1r A2r A3r A4r SINKS Epsilon
if sum(P(SINKS))<Epsilon*2
    Jac = Ar;
    if Omega1~=0
        Jac=Jac+real(exp(-i*Omega1*t-i*pi/2))*A1r;
    end
    if Omega2~=0
        Jac=Jac+real(exp(-i*Omega2*t-i*pi/2))*A2r;
    end
    if Omega3~=0
        Jac=Jac+real(exp(-i*Omega3*t-i*pi/2))*A3r;
    end
    if Omega4~=0
        Jac=Jac+real(exp(-i*Omega4*t-i*pi/2))*A4r;
    end
else
    Jac= 0*Ar;
end
end
