%%  FSP Example 1: Birth Death.
clear all
%% Birth Death process, approach 1.
close all
eps_min = 1e-6;     % Minimum allowable error
k = 50;             % Transcription rate
g = 1;              % Degradation rate
N = 10;             % First guess of the max number of RNA.
eps = 1;            % Current error.
tf = 100;             % final time

%% Define Multistate Model
N_States = 2;
Kr = [1 50];
Ktrans = [-20 1;...     % State Change reactions.
    20 -1]/1000;
%%
Deg = g*eye(N_States);  % Degradation RXNs
Transc = diag(Kr);      % Transcription reactions.

while  eps>eps_min
    Pinit = spalloc((N+1)*N_States+N_States,1,1);   % Initialize probability distribution.
    Pinit(1) = 1;
    
    %% Using for loops (SLOW)
    A = spalloc((N+1)*N_States+N_States,(N+1)*N_States+N_States,((N+1)*N_States+N_States)*5);
    for i=0:N
        Ind_States = [i*N_States+1:(i+1)*N_States];
        A(Ind_States,Ind_States) = A(Ind_States,Ind_States)+Ktrans-i*Deg-Transc;   % Flow out.
        A(Ind_States+N_States,Ind_States) = Transc;                      % Flow out due to production
        if i>=1
            A(Ind_States-N_States,Ind_States) = i*Deg;                % Flow out due to degradation
        end
    end
    
    %% Using EXPM (SLOW)
    Pf = expm(A*tf)*Pinit;
    
    eps = sum(Pf(end-N_States+1:end));
    if eps>eps_min
        N=ceil(N*1.1);
    end
end

P_RNA = zeros(N+1,1);
for j=1:N_States
    P_RNA = P_RNA + Pf(j:N_States:(end-N_States));
end

plot([0:N],P_RNA,'b')


