%%  FSP Example 1: Birth Death.
%% Birth Death process, approach 1.
close all
clear all

%% Specify Birth and Death Rates as functions of x.
% Be sure to define these for vector inputs (i.e., use .* and ./ and .^ )
% k_of_x = @(x)130;    % Transcription rate
k_of_x = @(x)10+150*x.^3/(150^3+x.^3);    % Transcription rate
g_of_x = @(x)1*x;   % Degradation rate

%% Specify initial probability distribution.
P0(1) = sparse(1);

%% Specify times at which to plot results.
time_points = linspace(0,1,10);
figure; clf;

%% Error tollerance, and initial projection.
eps_min = 1e-6;     % Minimum allowable error
N = 10;             % First guess of the max number of RNA.
eps = 1;            % Current error.

for tf = time_points(2:end);
    eps = 1;            % Current error.
    while  eps>eps_min
        P0(N+2,1) = 0;
        A = spalloc(N+2,N+2,(N+2)*3);
        for i=0:N
            k = k_of_x(i);
            g = g_of_x(i);
            A(i+1,i+1) =  -k-g;   % Flow out.
            A(i+2,i+1) = k;       % Flow in due to production
            if i>=1
                A(i,i+1) = g;     % Flow in due to degradation
            end
        end
        %% Using EXPM (SLOW)
        %     Pf = expm(A*tf)*Pinit;

        %% Using Expokit (Faster)
        Pf = mexpv(tf,A,P0);
        
        eps = Pf(end);
        if eps>eps_min
            N=ceil(N*1.2)
        end
    end
    plot([0:N],Pf(1:end-1),'b')
    hold on
end

%% Verify for Poisson Process at final time.
% k=k_of_x(1);
% g=g_of_x(1);
% Y = pdf('poiss',[0:N],k/g*(1-exp(-g*tf)));
% plot([0:N],Y,'r--')
% 

